function [position,power,s,AP] = fp_v(xmax,ymax,noOfFloor,floorHeight,tol,n_dev,n_ap)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
%       Department of Electronics and Communication Engineering        %
%                        WANG WENBO - 238970                           %
%                                                                      %
%                 function to design Fingerprints                      %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - xmax and ymax is the length and width of this building [m]         %
% - noOfFloor is the number of floors                                  %
% - floorHeight is the height spacing of floors                        %
% - tol is the upper bound of tolerance that can be achieved           %
% - fingerPrint is cell data, which the first row stands for the finger%
% location and the second row stands for AP and received power         %
%                                                                      %
% NOTE, inside fingerPrint, if power equal to 0, which means this      %
% position cannot hear that AP                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% initial
%AP=apDesign(xmax,ymax,noOfFloor,floorHeight);
AP={1;0;[xmax/2,ymax/2,floorHeight]};

sensitivity=-100;% assume the sensitivity of receiver is 100dBm

spacing=tol;% the spacing between 2 fingerprints
% assume the area formed by 4 neighbour fingerprints is square

% x axis coordinate of fingerprint
temp=mod(xmax,spacing)/2;
if temp==0
    x_fp=spacing/2:spacing:xmax;
else
    x_fp=temp:spacing:xmax;
end

% y axis coordinate of fingerprint
temp=mod(ymax,spacing)/2;
if temp==0
    y_fp=spacing/2:spacing:ymax;
else
    y_fp=temp:spacing:ymax;
end

s=length(y_fp);

% z axis coordinate of fingerprint
z_fp=0:floorHeight:floorHeight*(noOfFloor-1);

% form the fingerprint grid
% x-y plane
temp_x=repmat(x_fp,length(y_fp),1);
temp_x=temp_x(:);
temp_y=repmat(y_fp.',1,length(x_fp));
temp_y=temp_y(:);
fpGrid=repmat([temp_x,temp_y],noOfFloor,1);
% z plane
temp_z=repmat(z_fp,length(temp_x),1);
temp_z=temp_z(:);
fpGrid(:,3)=temp_z;
position=fpGrid;

% define the received power model, here for the same transmitter, we assume
% the transmitted power and path loss coefficient are the same, and the
% niose power is instant changing value, for the different APs, the
% transmitted power differs and the path loss coefficient differs.
Pr=@(x,x1,PT,n_ap,n) PT-10*n_ap.*log10(sqrt(sum((x-x1).^2)))+n;

%n_ap=2+(7-2)*rand(1,length(AP));% path loss coefficient modeled by Uniform
% distribution and in the interval [2,7]

temp_fp=zeros(length(fpGrid),1);

% form noise database
noise=randn(1,length(fpGrid)*1);
noise=(noise-mean(noise))./std(noise,1);
noise=n_dev*noise;
noise=reshape(noise,length(fpGrid),1);

for p=1:length(fpGrid)
    
    for pp=1:1
        
        temp_fp(p,pp)=Pr(fpGrid(p,:),AP{3,pp},AP{2,pp},n_ap(pp),noise(p,pp));
        
    end
    
end


% we assume the power level below sensitivity cannot be heard
temp_fp(temp_fp<sensitivity)=nan;

power=temp_fp;
end