function [B,I,est_pos,trk_pos,error] = estimation0(var,fingerPrint,track,n,algorithm)

% this estimated position calculated by Gaussian estimation, and then pick
% the first n position to calculate the average position, which is taken as
% the estimated position
%
% track is resampled track information,and the structure is{position;power}
% n, the mean value of the first n estimated fingerprint positions is taken
% as the estimated track positions 
% sigma, unit is dB
% algorithm, 2 is sum of probability, which can give better estimation
%
%case 1 is linear addition
%case 2 is logarithm addition
%case 3 is the linear addition of multiplication of probability and
%difference between heard power and fingerprint power
%
%you can add more case to test which algorithm is the opitimized
% 
% Author - WANG WENBO
% Department of Electronics and Communication Engineering

% initial
fp_pos=zeros(length(fingerPrint),3);
fp_power=zeros(length(fingerPrint),length(fingerPrint{2,1}));
trk_pos=zeros(length(track),3);
trk_power=zeros(length(track),length(track{2,1}));

% initialize fingerprint positions
for p=1:length(fp_pos)
    
    fp_pos(p,:)=fingerPrint{1,p};
    
end
clear p;
% initialize fingerPrint powers for each position
for p=1:length(fingerPrint)
    
    temp=fingerPrint{2,p};
    fp_power(p,:)=temp(2,:);
    
end
clear p temp;
% initialize track positions
for p=1:length(trk_pos)
    
    trk_pos(p,:)=track{1,p};
    
end
clear p;
% initialize track powers for each position
for p=1:length(track)
    
    temp=track{2,p};
    trk_power(p,:)=temp(2,:);
    
end
clear p temp;

% form fp power matrix
fpMatrix=repmat(fp_power,length(track),1);

% form track power matrix
trkMatrix=kron(trk_power,ones(length(fingerPrint),1));

% apply the algorithm
% function
p=@(a,b) (1/sqrt(2*pi*var))*exp(-(a-b).^2/(2*var));

% only use 1, this is sum of probability, with 1 and 2, this takes
% probability as weights of distance of power

switch algorithm
    case 1
        pr=p(fpMatrix,trkMatrix);
    case 2
        pr=p(fpMatrix,trkMatrix);
        pr(isnan(pr))=1e-20;
        pr=log10(pr);
    case 3
        pr=p(fpMatrix,trkMatrix);
        pr=pr.*abs(fpMatrix-trkMatrix);
end

%pr(isnan(pr))=1e-6;

%pr=log(p(fpMatrix,trkMatrix));
%pr=p(fpMatrix,trkMatrix); % 1
%pr=log(pr);
%pr=pr.*(fpMatrix-trkMatrix);
%pr=pr.*(trkMatrix-fpMatrix);
%pr=pr.*abs(fpMatrix-trkMatrix); % 2
%pr=pr.*10.^(abs(fpMatrix-trkMatrix));
%pr=log(pr);
est_temp=nansum(pr,2);
%{
[~,len]=size(pr);
templen=len-sum(isnan(pr),2);
templen=templen;

est_temp=est_temp.*templen;
%}
temp=reshape(est_temp,length(fingerPrint),length(track));
[B,I]=sort(temp,'descend');
est_n=I(1:n,:);% first 4 estimated fingerprint positions
est_n=est_n(:);

clear est_temp temp;

% calculate the estimated position by the average of first 4 estimated
% fingerprinr positions
temp=fp_pos(est_n,:);
temp_pos=reshape(temp,n,length(track)*3);
temp_ppos=sum(temp_pos)/n;
est_pos=reshape(temp_ppos,length(track),3);

clear est_n temp temp_pos temp_ppos;

error=sqrt(sum((est_pos-trk_pos).^2,2));

end

