%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% mapping                                                              %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %                  
% this script is used to calculate the power of each point in the track%
% note: 1.this script is not independent, in other words, it is highly %
%       dependent on the initial part!!! When you use this script, be  %
%       careful and modify it first so that it can be functional.      %
%       2.this script is a necessary step before estimation!           %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;

% initial
load('hybrid_model_12.mat');
load('fp_10_noise.mat');
var=10;

% sort AP and select the strongest one to experiment
ap=zeros(1,length(AP));
index=ap;

for pit=1:length(AP)
    
    ap(1,pit)=AP{2,pit};
    index(1,pit)=AP{1,pit};
    
end
clear pit;
%%{
ap_temp=reshape(ap,length(AP)/noOfFloor,noOfFloor);
index_temp=reshape(index,length(AP)/noOfFloor,noOfFloor);

[~,ind]=sort(ap_temp);
position=ind(end,:);

[a,b]=size(ap_temp);
pos=zeros(1,b);

clear ap_temp;

for ppit=1:b
    
    pos(ppit)=index_temp(position(ppit),ppit);
    
end
clear ppit;
%}


%tol=4; % the maximum deviation from real position is 3 meters
%noOfFloor=noFloor;
%fingerPrint = fpDesign(xmax,ymax,noOfFloor,floorHeight,tol);

t_simu=120*60; % simulation time is 240 minutes

fs=1; % resample frequency is 1 second
m=fs*Fs;

new_trace=trace_rss(:,1:m:t_simu*m+1);
%new_trace=trace(:,1:m:t_simu*m+1);

%{
% separate by floor, sort by z axis
t_temp=new_trace(3,:);
[~,I]=sort(t_temp);
new_trace=new_trace(:,I);
%}

%%{
% figure
figure;
plot3(new_trace(1,1:200:end),new_trace(2,1:200:end),new_trace(3,1:200:end),'k.');hold on;
plot3(new_trace(1,1:200:end),new_trace(2,1:200:end),new_trace(3,1:200:end),'m-');
plot3(new_trace(1,1),new_trace(2,1),new_trace(3,1),'ro','markersize',8);
plot3(new_trace(1,end),new_trace(2,end),new_trace(3,end),'bo','markersize',8);
grid on;hold off;
axis([0 xmax 0 ymax 0 noOfFloor*floorHeight]);
title(['simulation time is ',num2str(t_simu/60),' minutes']);

%}

% the positions after resample
track=cell(2,length(new_trace));
for p=1:length(new_trace)
    
    track{1,p}=new_trace(:,p);
    
end
clear p;
% the power of each position after resample
Pr=@(x,x1,PT,n_ap,n) PT-10*n_ap.*log10(sqrt(sum((x-x1).^2)))+n;

% noise deviation 10 dB
n_dev=sqrt(var);

% form noise database
noise=randn(1,length(new_trace)*length(AP));
noise=(noise-mean(noise))./std(noise,1);
noise=n_dev*noise;
noise=reshape(noise,length(new_trace),length(AP));

%% calculate the power of each position of the track

temp_fp=zeros(length(new_trace),length(AP));

for p=1:length(new_trace)
    
    for pp=1:length(AP)
        
        temp_fp(p,pp)=Pr(new_trace(:,p)',AP{3,pp},AP{2,pp},n_ap(pp),noise(p,pp));
        
    end
    
end

temp1=new_trace(3,:)<3;
temp2=new_trace(3,:)>=3&new_trace(3,:)<6;
temp3=new_trace(3,:)>=6&new_trace(3,:)<9;
x1=temp_fp(temp1,:);
x2=temp_fp(temp2,:);
x3=temp_fp(temp3,:);

[a1,b1]=size(x1);
[a2,b2]=size(x2);
[a3,b3]=size(x3);

% floor loss
temp_fp(temp1,:)=x1+repmat([zeros(1,7),ones(1,7)*(-5),ones(1,7)*(-10)],a1,1);
temp_fp(temp2,:)=x2+repmat([ones(1,7)*(-5),zeros(1,7),ones(1,7)*(-5)],a2,1);
temp_fp(temp3,:)=x3+repmat([ones(1,7)*(-10),ones(1,7)*(-5),zeros(1,7)],a3,1);

% we assume the power level below sensitivity cannot be heard
temp_fp(temp_fp<-100)=nan;

for p=1:length(new_trace)
    
    track{2,p}=[1:length(AP);temp_fp(p,:)];
    
end
clear p;

%%{
est_position=zeros(length(new_trace),3);
temp=zeros(3,length(fingerPrint));

% extract the position data from fingerPrint mat
for q=1:length(fingerPrint)
    
    temp(:,q)=fingerPrint{1,q}.';
    
end

clear q;

count=1;
for q=1:length(new_trace)
    
    temp_trace=repmat(new_trace(:,q),1,length(fingerPrint));
    dif=sum((temp_trace-temp).^2);
    [~,ind]=sort(dif);
    est_position(q,:)=temp(:,ind(1)).';
    
    est_power(count:count+1,:)=fingerPrint{2,ind(1)};
    count=count+2;
    
    clear dif;
    
end


% figure
tempa1=AP{3,pos(1)};tempa2=AP{3,pos(2)};tempa3=AP{3,pos(3)};
%tempa4=AP{3,pos(4)};tempa5=AP{3,pos(5)};
figure;
plot3(est_position(:,1),est_position(:,2),est_position(:,3),'k.');
grid on; hold on;
plot3(tempa1(1),tempa1(2),tempa1(3),'o',tempa2(1),tempa2(2),tempa2(3),'o',...
    tempa3(1),tempa3(2),tempa3(3),'o');
%,tempa4(1),tempa4(2),tempa4(3),'o',...
 %   tempa5(1),tempa5(2),tempa5(3),'o');
hold off;
axis([0 xmax 0 ymax 0 noOfFloor*floorHeight]);
title(['simulation time is ',num2str(t_simu/60),' minutes']);

% fingure
figure;
time=0:length(est_power)/2-1;
plot(time',est_power(2:2:end,pos(1)),time',est_power(2:2:end,pos(2)),time',est_power(2:2:end,pos(3)));
%,...
 %   time',est_power(2:2:end,pos(4)),time',est_power(2:2:end,pos(5)),'-');
hold on;

grid on; xlabel('time [s]'); ylabel('power [dB]');
title(['power level heard from AP ',num2str(1),' to AP ',num2str(3)]);
legend('AP1','AP2','AP3');
%,'AP4','AP5');

%plot(1139*ones(1,100),-100:0.9:-10-0.9,'--',2695*ones(1,100),-100:0.9:-10-0.9,'--');
    %4151*ones(1,100),-100:0.9:-10-0.9,'--',5521*ones(1,100),-100:0.9:-10-0.9,'--');
hold off;
%}

%save('rd_hahaha_track.mat','track','var');