%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Random Direction Mobility Model                                      %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %                  
% 2-D and 3-D models are included, this animate only shows the 2-D     %
% scenarios                                                            %
% note: 1.the displacement for each step is not constant               %
%       2.when storing the trajectory data of random direction mobility%
%       model, you should remove the '%' comment symbol. The trajectory%
%       data is stored in 3-D model way.                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;
path(path,'functions');
path(path, 'classes');
%% variables
% define x-y plane
xmax=50;ymax=50;

N=10*60*600;% simulation time
%N=5e3;

noOfFloor=3;% the number of floors
floorHeight=3;% the height spacing of floors

% fixed starting point
%{
x0=0;y0=0;z0=0;
%}

temp_3=0:floorHeight:floorHeight*(noOfFloor-1);

% random starting point
%%{
x0=xmax*rand;
y0=ymax*rand;
z0=temp_3(randi(length(temp_3)));
%}

% predefined starting point
%load('para2.mat');

Fs=10;  % sampling frequency [Hz]
t=1/Fs; % velocity changes every 1/Fs second
v_max=3;v_min=0.5;% the range of velocity
%v_min=0.89;v_max=1.083;

% use setVariable function to distribute the velocity variables within the
% range [v_min, v_max], the parameter 'normal' will be explained
% specifically in the definiton of setVaribale function
v=setVariable(v_min,v_max,'normal',N);
d=v*t;  % form the displacement set [m]

% the height dimension range
zmin=0;zmax=(noOfFloor-1)*floorHeight;

%%{
%% 2-D
figure;

% plotting the boundary...
h4=plot([0 xmax],[ymax ymax],'r-','linewidth',1.5); hold on;
plot([0 xmax],[0 0],'r-','linewidth',1.5);
plot([0 0],[0 ymax],'r-','linewidth',1.5);
plot([xmax xmax],[0 ymax],'r-','linewidth',1.5);

% the first point
trace(1,1)=x0;
trace(2,1)=y0;

% the new angle of user's direction after touching the boundary
r=-pi+2*pi*rand;

% set the counter
count=1;

% processing...
for n=1:N
    
    % plotting the starting point...
    h1=plot(x0,y0,'go','markersize',10,'linewidth',1.5);grid on;
    % set the axis
    axis('square',[-5 xmax+5 -5 ymax+5]);
    % compute the coordinate of next point according to the current point
    trace(1,count+1)=trace(1,count)+d(n)*cos(r);
    trace(2,count+1)=trace(2,count)+d(n)*sin(r);
    
    % the first 'if' is the case when the user is moving within the
    % simulation area boundary
    if trace(1,count+1)<xmax&&trace(1,count+1)>0&&trace(2,count+1)<ymax&&trace(2,count+1)>0
        % plotting the line connected the previous point and the current
        % point...
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-','linewidth',0.6);
        % the counter increases by 1
        count=count+1;
        
        % notice!! this following algorithm is different from the one from
        % the boundless simulation area mobility model
        
        % when the next point crosses the boundary, the heading direction
        % will change. But before this action, the user will touch the
        % boundary first
    elseif trace(1,count+1)>xmax&&trace(2,count+1)<0
        % touching the boundary...
        trace(1,count+1)=xmax;
        trace(2,count+1)=0;
        plot([trace(1,count),xmax],[trace(2,count),0],'k-','linewidth',0.6);
        plot(xmax,0,'ko','markersize',5,'markerface','k');
        
        % every time when the status of user changes, the counter increases
        % by 1
        count=count+1;
        
        % use the setVariable function to create new direction within the
        % possible angles range, in this case, it is pi/2 to pi
        r=setVariable(pi/2,pi,'normal',1);
        trace(1,count+1)=xmax+d(n)*cos(r);
        trace(2,count+1)=d(n)*sin(r);
        plot([xmax,trace(1,count+1)],[0,trace(2,count+1)],'k-','linewidth',0.6);        
        count=count+1;
    elseif trace(1,count+1)>xmax&&trace(2,count+1)>ymax
        trace(1,count+1)=xmax;
        trace(2,count+1)=ymax;
        plot([trace(1,count),xmax],[trace(2,count),ymax],'k-','linewidth',0.6);
        plot(xmax,ymax,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(pi,3*pi/2,'normal',1);
        trace(1,count+1)=xmax+d(n)*cos(r);
        trace(2,count+1)=ymax+d(n)*sin(r);
        plot([xmax,trace(1,n+1)],[ymax,trace(2,n+1)],'k-','linewidth',0.6);
        count=count+1;
    elseif trace(1,count+1)<0&&trace(2,count+1)>ymax
        trace(1,count+1)=0;
        trace(2,count+1)=ymax;
        plot([trace(1,count),0],[trace(2,count),ymax],'k-','linewidth',0.6);
        plot(0,ymax,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(3*pi/2,2*pi,'normal',1);
        trace(1,count+1)=d(n)*cos(r);
        trace(2,count+1)=ymax+d(n)*sin(r);
        plot([0,trace(1,count+1)],[ymax,trace(2,count+1)],'k-','linewidth',0.6);
        count=count+1;
    elseif trace(1,count+1)<0&&trace(2,count+1)<0
        trace(1,count+1)=0;
        trace(2,count+1)=0;
        plot([trace(1,count),0],[trace(2,count),0],'k-','linewidth',0.6);
        plot(0,0,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(0,pi/2,'normal',1);
        trace(1,count+1)=d(n)*cos(r);
        trace(2,count+1)=d(n)*sin(r);
        plot([0,trace(1,count+1)],[0,trace(2,count+1)],'k-','linewidth',0.6);
        count=count+1;
    elseif trace(1,count+1)>=xmax&&trace(2,count+1)>=0&&trace(2,count+1)<=ymax
        % the y-axis intersection point
        y=((xmax-trace(1,count))*trace(2,count+1)+(trace(1,count+1)-xmax)*trace(2,count))...
            /(trace(1,count+1)-trace(1,count));
        trace(1,count+1)=xmax;
        trace(2,count+1)=y;
        plot([trace(1,count),xmax],[trace(2,count),y],'k-','linewidth',0.6);
        plot(xmax,y,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(pi/2,3*pi/2,'normal',1);
        trace(1,count+1)=xmax+d(n)*cos(r);
        trace(2,count+1)=y+d(n)*sin(r);
        plot([xmax,trace(1,count+1)],[y,trace(2,count+1)],'k-','linewidth',0.6);
        count=count+1;
    elseif trace(1,count+1)<=0&&trace(2,count+1)>=0&&trace(2,count+1)<=ymax
        y=((0-trace(1,count))*trace(2,count+1)+trace(1,count+1)*trace(2,count))...
            /(trace(1,count+1)-trace(1,count));
        trace(1,count+1)=0;
        trace(2,count+1)=y;
        plot([trace(1,n),0],[trace(2,n),y],'k-','linewidth',0.6);
        plot(0,y,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(3*pi/2,5*pi/2,'normal',1);
        trace(1,count+1)=d(n)*cos(r);
        trace(2,count+1)=y+d(n)*sin(r);
        plot([0,trace(1,count+1)],[y,trace(2,count+1)],'k-','linewidth',0.6);
        count=count+1;
    elseif trace(2,count+1)>=ymax&&trace(1,count+1)>=0&&trace(1,count+1)<=xmax
        x=((ymax-trace(2,count))*trace(1,count+1)+(trace(2,count+1)-ymax)*trace(1,count))...
            /(trace(2,count+1)-trace(2,count));
        trace(1,count+1)=x;
        trace(2,count+1)=ymax;
        plot([trace(1,count),x],[trace(2,count),ymax],'k-','linewidth',0.6);
        plot(x,ymax,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(pi,2*pi,'normal',1);
        trace(1,count+1)=x+d(n)*cos(r);
        trace(2,count+1)=ymax+d(n)*sin(r);
        plot([x,trace(1,count+1)],[ymax,trace(2,count+1)],'k-','linewidth',0.6);
        count=count+1;
    elseif trace(2,count+1)<=0&&trace(1,count+1)>=0&&trace(1,count+1)<=xmax
        x=((0-trace(2,count))*trace(1,count+1)+trace(2,count+1)*trace(1,count))...
            /(trace(2,count+1)-trace(2,count));
        trace(1,count+1)=x;
        trace(2,count+1)=0;
        plot([trace(1,count),x],[trace(2,count),0],'k-','linewidth',0.6);
        plot(x,0,'ko','markersize',5,'markerface','k');
        count=count+1;
        r=setVariable(0,pi,'normal',1);
        trace(1,count+1)=x+d(n)*cos(r);
        trace(2,count+1)=d(n)*sin(r);
        h3=plot([x,trace(1,count+1)],[0,trace(2,count+1)],'k-','linewidth',0.6);
        count=count+1;
    end
    % open an animate
    drawnow;
end

h2=plot(trace(1,end),trace(2,end),'ro','markersize',10,'linewidth',1.5);

legend([h1,h2,h3,h4],'starting point','terminal point','track','boundary');
xlabel('x [m]');
ylabel('y [m]');

hold off;
%title(['Random Direction Mobility Mode','total ',num2str(N),' moves']);
%}

%{
%% 3-D
% the height dimension is less important than the x-y dimension, thereby it
% is modeled by the thirdDi function. In brief, the thirdDi function simply
% generates variables within a range by following a certain distribution
trace(3,:) = thirdDi(length(trace(1,:))-1,t,z0,zmin,zmax);
%%}

trace_rss=trace;

save('random_direction_25.mat');
%}