function variable = setVariable(min,max,distribution,n)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% form the variable set/sets through uniform distribution/normal       %
% distribution                                                         %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% - max and min are the upper and lower bound of variable respectively %
% - distribution is the way of variable distribution                   %
% - n is the number of total needed variable                           %
%                                                                      %
% here we assume all variables fall into confidential interval [-1.96, %
% 1.96] in the normal distribution, which the confidence level is 95%  %
%                                                                      %
% note: 1.using this function (parameter is set 'normal'), we expand/  %
%       narrow the sample space into [-1.96, 1.96]. This is based on   %
%       the assumption that the upper/lower limit of the variable set  %
%       we gave has 0.95 probability in the real observation           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch distribution
    case 'uniform'
        variable=min+(max-min)*rand(1,n);%uniform distribution
    case 'normal'
        temp=randn(1,n*10);%E=0,det=1,normal distribution
        temp=temp.*(temp>=-1.96&temp<=1.96);
        [~,~,temp]=find(temp);
        temp=temp(1:n);
        average=(max+min)/2;
        a=(max-average)/1.96;
        variable_temp=temp*a;
        variable=variable_temp+average;
end

end