function z = thirdDi(amount,timeEachStep,z0,ground,sky)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% form the height dimension                                            %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% - z is the 3rd dimension, can be vector or singular                  %
%                                                                      %
% - amount is the total number of sampled positions                    %
%                                                                      %
% - timeEachStep is the time of each move[unit sencond], here the      % 
% meaning of this variable is to evaluate the zChange value, in the 3rd%
% dimension, the distance between two moves is named zChange           %
%                                                                      %
% - z0 is the initial value of 3rd dimension                           %
%                                                                      %
% - ground is the lowest limit user can reach                          %
%                                                                      %
% - sky is the highest limit user can reach                            %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% assume 1 second human being can move below 3 meters height
temp=3*timeEachStep;

% under normal circumstances, users should not walk into either the ground
% or the sky, so the given candidate 3rd dimension value should be far
% bigger than the needed
zChange=-temp+2*temp*rand(1,amount*100);

% initial
z=zeros(1,amount+1);
z(1,1)=z0;
count=1;

for n=1:amount
    z(1,n+1)=z(1,n)+zChange(count);
    count=count+1;
    
    % this is to prevent users walking into the ground
    while z(1,n+1)<ground
        z(1,n+1)=z(1,n)+zChange(count);
        count=count+1;
        if z(1,n+1)>=ground
            break;
        end
    end
    
    % this is to prevent users walking into the sky
    while z(1,n+1)>sky
        z(1,n+1)=z(1,n)+zChange(count);
        count=count+1;
        if z(1,n+1)<=sky
            break;
        end
    end
end


end